package java.time;

import java.time.temporal.Temporal;
import java.time.temporal.ChronoUnit;
import java.time.temporal.TemporalQueries;
import java.time.temporal.TemporalQuery;
import java.time.temporal.UnsupportedTemporalTypeException;
import java.time.temporal.ValueRange;
import java.time.temporal.TemporalField;
import java.time.format.DateTimeFormatterBuilder;
import java.util.Locale;
import java.time.format.TextStyle;
import java.time.temporal.ChronoField;
import java.time.temporal.TemporalAdjuster;
import java.time.temporal.TemporalAccessor;

public enum DayOfWeek implements TemporalAccessor, TemporalAdjuster
{
    MONDAY, 
    TUESDAY, 
    WEDNESDAY, 
    THURSDAY, 
    FRIDAY, 
    SATURDAY, 
    SUNDAY;
    
    private static final DayOfWeek[] ENUMS;
    
    public static DayOfWeek of(int i) {
        if (i < 1 || i > 7) {
            throw new DateTimeException("Invalid value for DayOfWeek: " + i);
        }
        return DayOfWeek.ENUMS[i - 1];
    }
    
    public static DayOfWeek from(TemporalAccessor obj) {
        if ((obj instanceof DayOfWeek)) {
            return (DayOfWeek)obj;
        }
        try {
            return of(obj.get(ChronoField.DAY_OF_WEEK));
        }
        catch (DateTimeException ex) {
            throw new DateTimeException("Unable to obtain DayOfWeek from TemporalAccessor: " + obj + " of type " + obj.getClass().getName(), ex);
        }
    }
    
    public int getValue() {
        return this.ordinal() + 1;
    }
    
    public String getDisplayName(TextStyle textStyle, Locale locale) {
        return new DateTimeFormatterBuilder().appendText(ChronoField.DAY_OF_WEEK, textStyle).toFormatter(locale).format(this);
    }
    
    @Override
    public boolean isSupported(TemporalField temporalField) {
        if ((temporalField instanceof ChronoField)) {
            return temporalField == ChronoField.DAY_OF_WEEK;
        }
        return temporalField != null && temporalField.isSupportedBy(this);
    }
    
    @Override
    public ValueRange range(TemporalField field) {
        if (field == ChronoField.DAY_OF_WEEK) {
            return field.range();
        }
        return super.range(field);
    }
    
    @Override
    public int get(TemporalField field) {
        if (field == ChronoField.DAY_OF_WEEK) {
            return this.getValue();
        }
        return super.get(field);
    }
    
    @Override
    public long getLong(TemporalField obj) {
        if (obj == ChronoField.DAY_OF_WEEK) {
            return (long)this.getValue();
        }
        if ((obj instanceof ChronoField)) {
            throw new UnsupportedTemporalTypeException("Unsupported field: " + obj);
        }
        return obj.getFrom(this);
    }
    
    public DayOfWeek plus(long n) {
        return DayOfWeek.ENUMS[(this.ordinal() + ((int)(n % 7L) + 7)) % 7];
    }
    
    public DayOfWeek minus(long n) {
        return this.plus(-(n % 7L));
    }
    
    @Override
    public <R> R query(TemporalQuery<R> query) {
        if (query == TemporalQueries.precision()) {
            return ChronoUnit.DAYS;
        }
        return super.query(query);
    }
    
    @Override
    public Temporal adjustInto(Temporal temporal) {
        return temporal.with(ChronoField.DAY_OF_WEEK, (long)this.getValue());
    }
    
    static {
        ENUMS = values();
    }
}
